//*****************************************************************************
//
// LTC6946-3 Signal Generator
//
// Talks to the Linear Technology LTC6946-3 demo board (minus its SPI interface
// chip which was a bit flakey). Allows the user to set a frequency. Checks the
// user selected frequency, and if it can be generated by the LTC6946-3,
// calculates the magic numbers for the LTC6946-3 and then programs them into
// the appropriate registers via SPI.
//
// Project is still called "hello_widget" because changing a project name
// in CCS is far from an easy task. And I'm lazy.
//
// In terms of display screens there are only 3. The first screen displays
// the current frequency. The second screen provides a keypad where the user
// can enter a new frequency. The third screen displays an error message informing
// the user of the allowable frequency range if an invalid frequency is entered.
//
// Copyright (c) Frank Van Hooft 2012
//
//*****************************************************************************


//*****************************************************************************
//
// hello_widget.c - Simple hello world example using
//
// Copyright (c) 2008-2012 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
// 
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of revision 9107 of the RDK-IDM-L35 Firmware Package.
//
//*****************************************************************************

#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "inc/hw_nvic.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/systick.h"
#include "driverlib/uart.h"
#include "stdio.h"
#include "string.h"
#include "utils/uartstdio.h"
#include "drivers/sound.h"
#include "driverlib/gpio.h"
#include "grlib/grlib.h"
#include "grlib/widget.h"
#include "grlib/canvas.h"
#include "grlib/pushbutton.h"
#include "drivers/kitronix320x240x16_ssd2119.h"
#include "drivers/touch.h"
#include "utils/softssi.h"
#include "driverlib/ssi.h"





//*****************************************************************************
//
//! \addtogroup example_list
//! <h1>Hello using Widgets (hello_widget)</h1>
//!
//! A very simple ``hello world'' example written using the Stellaris Graphics
//! Library widgets.  It displays two buttons. The first, when pressed, toggles
//! display of the words ``Hello World!'' on the screen.  The second transfers
//! control to the serial boot loader to perform a software update.  This may
//! be used as a starting point for more complex widget-based applications.
//!
//! This application supports remote software update over serial using the
//! LM Flash Programmer application.  The application transfers control to the
//! boot loader whenever the ``Update Software'' button is pressed to allow a
//! new image to be downloaded.  The LMFlash serial data rate must be set to
//! 115200bps and the "Program Address Offset" to 0x800.
//!
//! UART0, which is connected to the 6 pin header on the underside of the
//! IDM-L35 RDK board (J8), is configured for 115200bps, and 8-n-1 mode.  The
//! USB-to-serial cable supplied with the IDM-L35 RDK may be used to connect
//! this TTL-level UART to the host PC to allow firmware update.
//
//*****************************************************************************


#define MAXCHARS	9			/* max number of characters the user can enter at keypad */
								// be careful increasing it - you can mess up UpdateFrequency() and pow10[]

// Minimum and maximum allowable frequency. If you change these, make certain to also
// change the text string in the g_s3Error3 canvas, which informs the user of the range.
// To fit a 32-bit integer, these are the frequencies divided by 4. Hence 5.79 GHz
// becomes 1.4475 GHz which is 1447500000 (or hex 0x564718e0)
// (This means our math is only accurate to 4 Hz, not 1 Hz. That's ok.)
// Note there's a gap in the output coverage; 2.85 GHz - 3.6 GHz cannot be generated.
#define MIN_FREQ 	160000000
#define MAX_FREQ	1447500000
#define MIN_GAP_FREQ 712500000
#define MAX_GAP_FREQ 900000000

#define uint64		unsigned long long

// Set this if you want to use the Soft SPI functions. Comment this out to use hardware SPI.
// #define USE_SOFT_SPI	1


// These copied from driverlib/pin_map.h because for some reason they weren't being included correctly
#define GPIO_PA2_SSI0CLK        0x00000801
#define GPIO_PA3_SSI0FSS        0x00000C01
#define GPIO_PA4_SSI0RX         0x00001001
#define GPIO_PA5_SSI0TX         0x00001401


//*****************************************************************************
//
// Global variables. Mostly strings that are edited by one function but drawn
// in the message queue handler (which is in the main loop) and hence need to
// be globally accessible.
//
//*****************************************************************************
char s_frequency[MAXCHARS+10];		// string holding current frequency
unsigned int charcount = 0;			// number of characters entered by user on keypad, excluding \0
char keypadbuff[MAXCHARS+10];		// store ASCII characters of user-entered frequency

// An array of precomputed values of 10 to the power of 0, 1, 2, 3, etc
const int pow10[] = {1, 10, 100, 1000, 10000, 100000, 1000000, 10000000, 100000000, 1000000000};


#if USE_SOFT_SPI
//*****************************************************************************
//
// The persistent state of the SoftSSI (Soft SPI) peripheral.
//
//*****************************************************************************
tSoftSSI g_sSoftSSI;

//*****************************************************************************
//
// The Soft SPI transmit and receive FIFO buffers.
//
//*****************************************************************************
unsigned short g_pusTxBuffer[16];
unsigned short g_pusRxBuffer[16];
#endif


//*****************************************************************************
//
// The sound effect that is played when a key is pressed.
//
//*****************************************************************************
static const unsigned short g_pusKeyClick[] =
{
    0, G5,
    15, SILENCE
};

//*****************************************************************************
//
// Forward reference to various widget structures.
//
//*****************************************************************************
extern tCanvasWidget g_s1Background;
extern tCanvasWidget g_s1Heading;
extern tPushButtonWidget g_s1SetFreqBtn;
extern tCanvasWidget g_s1CurrFreq;
extern tCanvasWidget g_s1ShowFreq;

extern tCanvasWidget g_s2Background;
extern tCanvasWidget g_s2EntFreq;
extern tCanvasWidget g_s2UserEntry;
extern tPushButtonWidget g_s2Key1;
extern tPushButtonWidget g_s2Key2;
extern tPushButtonWidget g_s2Key3;
extern tPushButtonWidget g_s2Key4;
extern tPushButtonWidget g_s2Key5;
extern tPushButtonWidget g_s2Key6;
extern tPushButtonWidget g_s2Key7;
extern tPushButtonWidget g_s2Key8;
extern tPushButtonWidget g_s2Key9;
extern tPushButtonWidget g_s2Key0;
extern tPushButtonWidget g_s2KeyPeriod;
extern tPushButtonWidget g_s2KeyBackspace;
extern tPushButtonWidget g_s2KeyMHz;
extern tPushButtonWidget g_s2KeyGHz;
extern tPushButtonWidget g_s2KeyCancel;

extern tCanvasWidget g_s3Background;
extern tCanvasWidget g_s3Error1;
extern tCanvasWidget g_s3Error2;
extern tCanvasWidget g_s3Error3;
extern tCanvasWidget g_s3Error4;
extern tCanvasWidget g_s3Error5;
extern tPushButtonWidget g_s3OKBtn;


//*****************************************************************************
//
// Forward references to the button press handlers.
//
//*****************************************************************************
void OnSetFreqPress(tWidget *pWidget);
void OnOKPress(tWidget *pWidget);
void KeyPadPress (tWidget *pWidget);


//*****************************************************************************
//
// SCREEN 1 - default display. Shows the current frequency.
//
//*****************************************************************************

//*****************************************************************************
//
// The canvas widget acting as the background to the display.
//
//*****************************************************************************
Canvas(g_s1Background, WIDGET_ROOT, 0, &g_s1Heading,
       &g_sKitronix320x240x16_SSD2119, 0, 0, 320, 240,
       CANVAS_STYLE_FILL, ClrBlack, 0, 0, 0, 0, 0, 0);

//*****************************************************************************
//
// The canvas widget containing the application title. This is the top of the
// first page (shown at startup and when the unit is running).
//
//*****************************************************************************
Canvas(g_s1Heading, &g_s1Background, 0, &g_s1CurrFreq,
       &g_sKitronix320x240x16_SSD2119, 0, 0, 320, 25,
       (CANVAS_STYLE_FILL | CANVAS_STYLE_TEXT),
       ClrDarkBlue, ClrWhite, ClrWhite, g_pFontCm22, "LTC6946 Signal Generator", 0, 0);

//*****************************************************************************
//
// The canvas widget used to display the "Current Frequency:" string.
//
//*****************************************************************************
Canvas(g_s1CurrFreq, &g_s1Heading, 0, &g_s1ShowFreq,
       &g_sKitronix320x240x16_SSD2119, 0, 60, 320, 40,
       (CANVAS_STYLE_FILL | CANVAS_STYLE_TEXT),
       ClrBlack, 0, ClrWhite, g_pFontCmss24, "Current Frequency:", 0, 0);

//*****************************************************************************
//
// The canvas widget used to display the "1.345 GHz" (or whatever) string.
// It displays the current frequency. Text currently defaults to displaying 0 Hz.
//
//*****************************************************************************
Canvas(g_s1ShowFreq, &g_s1CurrFreq, 0, &g_s1SetFreqBtn,
       &g_sKitronix320x240x16_SSD2119, 0, 100, 320, 40,
       (CANVAS_STYLE_FILL | CANVAS_STYLE_TEXT),
       ClrBlack, 0, ClrWhite, g_pFontCmss24, s_frequency, 0, 0);

//*****************************************************************************
//
// The button used to display the "Set Frequency" pushbutton. Pressing this button
// causes the second page to be displayed (ie the page where a new frequency can
// be entered)
//
//*****************************************************************************
RectangularButton(g_s1SetFreqBtn, &g_s1ShowFreq, 0, 0,
                  &g_sKitronix320x240x16_SSD2119, 60, 180, 200, 40,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "Set Frequency", 0, 0, 0, 0, OnSetFreqPress);



//*****************************************************************************
//
// SCREEN 2 - Keypad to allow user to enter new frequency.
//
//*****************************************************************************

//*****************************************************************************
//
// The canvas widget acting as the background to the display.
//
//*****************************************************************************
Canvas(g_s2Background, WIDGET_ROOT, 0, &g_s2EntFreq,
       &g_sKitronix320x240x16_SSD2119, 0, 0, 320, 240,
       CANVAS_STYLE_FILL, ClrBlack, 0, 0, 0, 0, 0, 0);

//*****************************************************************************
//
// The canvas widget saying "Enter Frequency:"
//
//*****************************************************************************
Canvas(g_s2EntFreq, &g_s2Background, 0, &g_s2UserEntry,
       &g_sKitronix320x240x16_SSD2119, 0, 0, 320, 40,
       (CANVAS_STYLE_FILL | CANVAS_STYLE_TEXT),
       ClrBlack, 0, ClrWhite, g_pFontCmss24, "Enter New Frequency:", 0, 0);

//*****************************************************************************
//
// The canvas widget displaying what the user is entering. Starts off blank,
// and is added to as the user presses keypad keys. keypadbuff is initialized when
// screen 2 is displayed.
//
//*****************************************************************************
Canvas(g_s2UserEntry, &g_s2EntFreq, 0, &g_s2Key1,
       &g_sKitronix320x240x16_SSD2119, 0, 35, 320, 40,
       (CANVAS_STYLE_FILL | CANVAS_STYLE_TEXT),
       ClrBlack, 0, ClrWhite, g_pFontCmss24, keypadbuff, 0, 0);

//*****************************************************************************
//
// The keypad buttons for keys 1 - 9 and 0.
//
//*****************************************************************************
RectangularButton(g_s2Key1, &g_s2UserEntry, 0, &g_s2Key2,
                  &g_sKitronix320x240x16_SSD2119, 16, 80, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "1", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2Key2, &g_s2Key1, 0, &g_s2Key3,
                  &g_sKitronix320x240x16_SSD2119, 92, 80, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "2", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2Key3, &g_s2Key2, 0, &g_s2Key4,
                  &g_sKitronix320x240x16_SSD2119, 168, 80, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "3", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2Key4, &g_s2Key3, 0, &g_s2Key5,
                  &g_sKitronix320x240x16_SSD2119, 16, 120, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "4", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2Key5, &g_s2Key4, 0, &g_s2Key6,
                  &g_sKitronix320x240x16_SSD2119, 92, 120, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "5", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2Key6, &g_s2Key5, 0, &g_s2Key7,
                  &g_sKitronix320x240x16_SSD2119, 168, 120, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "6", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2Key7, &g_s2Key6, 0, &g_s2Key8,
                  &g_sKitronix320x240x16_SSD2119, 16, 160, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "7", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2Key8, &g_s2Key7, 0, &g_s2Key9,
                  &g_sKitronix320x240x16_SSD2119, 92, 160, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "8", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2Key9, &g_s2Key8, 0, &g_s2Key0,
                  &g_sKitronix320x240x16_SSD2119, 168, 160, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "9", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2Key0, &g_s2Key9, 0, &g_s2KeyPeriod,
                  &g_sKitronix320x240x16_SSD2119, 16, 200, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "0", 0, 0, 0, 0, KeyPadPress);

//*****************************************************************************
//
// The keypad buttons for non-number keys (period, backspace, MHz, GHz & Cancel)
//
//*****************************************************************************
RectangularButton(g_s2KeyPeriod, &g_s2Key0, 0, &g_s2KeyBackspace,
                  &g_sKitronix320x240x16_SSD2119, 92, 200, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, ".", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2KeyBackspace, &g_s2KeyPeriod, 0, &g_s2KeyMHz,
                  &g_sKitronix320x240x16_SSD2119, 168, 200, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "BK", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2KeyMHz, &g_s2KeyBackspace, 0, &g_s2KeyGHz,
                  &g_sKitronix320x240x16_SSD2119, 244, 80, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "MHz", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2KeyGHz, &g_s2KeyMHz, 0, &g_s2KeyCancel,
                  &g_sKitronix320x240x16_SSD2119, 244, 120, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "GHz", 0, 0, 0, 0, KeyPadPress);

RectangularButton(g_s2KeyCancel, &g_s2KeyGHz, 0, 0,
                  &g_sKitronix320x240x16_SSD2119, 244, 160, 60, 35,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss20b, "CANC", 0, 0, 0, 0, KeyPadPress);


//*****************************************************************************
//
// SCREEN 3 - Error screen. Tell user the acceptable frequency range.
//
//*****************************************************************************

//*****************************************************************************
//
// The canvas widget acting as the background to the display.
//
//*****************************************************************************
Canvas(g_s3Background, WIDGET_ROOT, 0, &g_s3Error1,
       &g_sKitronix320x240x16_SSD2119, 0, 0, 320, 240,
       CANVAS_STYLE_FILL, ClrBlack, 0, 0, 0, 0, 0, 0);

//*****************************************************************************
//
// The canvas widgets used to display the Error message strings
//
//*****************************************************************************
Canvas(g_s3Error1, &g_s3Background, 0, &g_s3Error2,
       &g_sKitronix320x240x16_SSD2119, 0, 10, 320, 40,
       (CANVAS_STYLE_FILL | CANVAS_STYLE_TEXT),
       ClrBlack, 0, ClrWhite, g_pFontCmss24b, "ERROR", 0, 0);

Canvas(g_s3Error2, &g_s3Error1, 0, &g_s3Error3,
       &g_sKitronix320x240x16_SSD2119, 0, 60, 320, 30,
       (CANVAS_STYLE_FILL | CANVAS_STYLE_TEXT),
       ClrBlack, 0, ClrWhite, g_pFontCmss24, "Frequency range is", 0, 0);

Canvas(g_s3Error3, &g_s3Error2, 0, &g_s3Error4,
       &g_sKitronix320x240x16_SSD2119, 0, 90, 320, 30,
       (CANVAS_STYLE_FILL | CANVAS_STYLE_TEXT),
       ClrBlack, 0, ClrWhite, g_pFontCmss24, "640 MHz - 2.85 GHz", 0, 0);

Canvas(g_s3Error4, &g_s3Error3, 0, &g_s3Error5,
       &g_sKitronix320x240x16_SSD2119, 0, 120, 320, 30,
       (CANVAS_STYLE_FILL | CANVAS_STYLE_TEXT),
       ClrBlack, 0, ClrWhite, g_pFontCmss24, "3.6 GHz - 5.79 GHz", 0, 0);

Canvas(g_s3Error5, &g_s3Error4, 0, &g_s3OKBtn,
       &g_sKitronix320x240x16_SSD2119, 0, 150, 320, 30,
       (CANVAS_STYLE_FILL | CANVAS_STYLE_TEXT),
       ClrBlack, 0, ClrWhite, g_pFontCmss24, "or 0", 0, 0);

//*****************************************************************************
//
// The button used to display the "OK" pushbutton. Pressing this button
// causes the first page to be displayed (ie the "current frequency" page)
//
//*****************************************************************************
RectangularButton(g_s3OKBtn, &g_s3Error4, 0, 0,
                  &g_sKitronix320x240x16_SSD2119, 60, 190, 200, 40,
                  (PB_STYLE_OUTLINE | PB_STYLE_TEXT_OPAQUE | PB_STYLE_TEXT |
                   PB_STYLE_FILL | PB_STYLE_RELEASE_NOTIFY),
                   ClrDarkBlue, ClrBlue, ClrWhite, ClrWhite,
                   g_pFontCmss22b, "OK", 0, 0, 0, 0, OnOKPress);




//*****************************************************************************
//
// The error routine that is called if the driver library encounters an error.
//
//*****************************************************************************
#ifdef DEBUG
void
__error__(char *pcFilename, unsigned long ulLine)
{
}
#endif



//*****************************************************************************
//
// The interrupt handler for the SysTick interrupt. Used by soft SPI functions.
//
//*****************************************************************************
void SysTickIntHandler(void)
{
    //
    // Call the SoftSSI timer tick.
    //
#if USE_SOFT_SPI
	SoftSSITimerTick(&g_sSoftSSI);
#endif
}



//*****************************************************************************
//
// Initialise the SPI. This uses the soft SPI functions, which require the
// SysTick timer. This init routine does everything EXCEPT enable the SysTick
// interrupt. This is so we only need to suffer the CPU overhead of the SysTick
// interrupt when we're actually needing to perform SPI transactions. Hence the
// programming model is:
//   -- init SPI  (this function)
// Then each time you want to perform some SPI transfers:
//   -- enable SysTick interrupt using SysTickEnable()
//   -- perform your SPI reads and writes
//   -- disable SysTick interrupt using SysTickDisable()
//
//
//*****************************************************************************
void Init_SPI (void)
{

#if USE_SOFT_SPI
    //
    // For this example SoftSSI is used with PortA[5:2].
	// GPIO port A needs to be enabled so these pins can be used.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);

    //
    // Configure the SoftSSI module. The pins are assigned as follows:
    //      PA2 - SoftSSICLK
    //      PA3 - SoftSSIFss
    //      PA4 - SoftSSIRx
    //      PA5 - SoftSSITx
    //
    memset(&g_sSoftSSI, 0, sizeof(g_sSoftSSI));
    SoftSSIClkGPIOSet(&g_sSoftSSI, GPIO_PORTA_BASE, GPIO_PIN_2);
    SoftSSIFssGPIOSet(&g_sSoftSSI, GPIO_PORTA_BASE, GPIO_PIN_3);
    SoftSSIRxGPIOSet(&g_sSoftSSI, GPIO_PORTA_BASE, GPIO_PIN_4);
    SoftSSITxGPIOSet(&g_sSoftSSI, GPIO_PORTA_BASE, GPIO_PIN_5);
    SoftSSIRxBufferSet(&g_sSoftSSI, g_pusRxBuffer,
                       sizeof(g_pusRxBuffer) / sizeof(g_pusRxBuffer[0]));
    SoftSSITxBufferSet(&g_sSoftSSI, g_pusTxBuffer,
                       sizeof(g_pusTxBuffer) / sizeof(g_pusTxBuffer[0]));

    //
    // Configure the SoftSSI module.  Use idle clock level low and active low
    // clock (mode 0) and 16-bit data.
    //
    SoftSSIConfigSet(&g_sSoftSSI, SOFTSSI_FRF_MOTO_MODE_0, 16);

    //
    // Enable the SoftSSI module.
    //
    SoftSSIEnable(&g_sSoftSSI);

    //
    // Configure SysTick to provide an interrupt at a 10 KHz rate.  This is
    // used to control the clock rate of the SoftSSI module; the clock rate of
    // the SoftSSI Clk signal will be 1/2 the interrupt rate.
    //
    SysTickPeriodSet(SysCtlClockGet() / 10000);
    SysTickIntEnable();

    // Note that calling SysTickEnable() at this point would cause the 10 kHz SysTick
    // interrupt to start triggering. To reduce the impact of this CPU overhead, only
    // enable the SysTick interrupt when you actually need to perform SPI transfers.
#else

    // Initialise Hardware SPI port. Pinout is the same as above. We use SSI port 1.

    // The SSI0 peripheral must be enabled for use.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_SSI0);

    //
    // For this example SSI0 is used with PortA[5:2].
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);

    //
    // Configure the pin muxing for SSI0 functions on port A2, A3, A4, and A5.
    //
    GPIOPinConfigure(GPIO_PA2_SSI0CLK);
    GPIOPinConfigure(GPIO_PA3_SSI0FSS);
    GPIOPinConfigure(GPIO_PA4_SSI0RX);
    GPIOPinConfigure(GPIO_PA5_SSI0TX);

    //
    // Configure the GPIO settings for the SSI pins.  This function also gives
    // control of these pins to the SSI hardware. The pins are assigned as follows:
    //      PA5 - SSI0Tx
    //      PA4 - SSI0Rx
    //      PA3 - SSI0Fss
    //      PA2 - SSI0CLK
    //
    GPIOPinTypeSSI(GPIO_PORTA_BASE, GPIO_PIN_5 | GPIO_PIN_4 | GPIO_PIN_3 |
                   GPIO_PIN_2);

    //
    // Configure and enable the SSI port for SPI master mode.  Use SSI0,
    // system clock supply, idle clock level low and active low clock in
    // freescale SPI mode, master mode, 50 kHz SSI frequency, and 16-bit data.
    //
    SSIConfigSetExpClk(SSI0_BASE, SysCtlClockGet(), SSI_FRF_MOTO_MODE_0,
                       SSI_MODE_MASTER, 50000, 16);

    //
    // Enable the SSI0 module.
    //
    SSIEnable(SSI0_BASE);
#endif
}




//*****************************************************************************
//
// SPI Read and Write Functions. Look at page 18 of the LTC6946 datasheet
// to see what the SPI data needs to look like. Each takes a register address
// (8 bits) as a parameter. The read function returns 8 bits of data, and the
// write function accepts 8 bits of data as a parameter and returns 0 on success.
//
//*****************************************************************************
unsigned short SPI_Write (unsigned short addr, unsigned short data)
{
#if USE_SOFT_SPI
	SysTickEnable();

	SoftSSIDataPut(&g_sSoftSSI, ((addr<<9)&0xfe00) | (data&0xff));		// SPI: LSB of addr must be 0 for a write

	while(SoftSSIBusy(&g_sSoftSSI));					// wait for SPI to finish transmitting
	SysTickDisable();
#else
	// use hardware SPI

    //
    // Send the data using the "blocking" put function.  This function
    // will wait until there is room in the send FIFO before returning.
    // This allows you to assure that all the data you send makes it into
    // the send FIFO.
    //
    SSIDataPut(SSI0_BASE, ((addr<<9)&0xfe00) | (data&0xff));		// SPI: LSB of addr must be 0 for a write
    while(SSIBusy(SSI0_BASE));		// Wait until SSI0 is done transferring all the data in the transmit FIFO.

#endif

	return 0;
}

unsigned short SPI_Read (unsigned short addr)
{
	unsigned long ulDataRx[8];							// temporary SPI receive data location

#if USE_SOFT_SPI
	SysTickEnable();

	while (SoftSSIDataGetNonBlocking(&g_sSoftSSI, ulDataRx));	// read Rx FIFO until it's empty (ie dump junk)

	SoftSSIDataPut(&g_sSoftSSI, (addr<<9)|0x0100);		// SPI: LSB of addr must be 1 for a read

	while(SoftSSIBusy(&g_sSoftSSI));					// wait for SPI to finish transmitting

	SoftSSIDataGetNonBlocking(&g_sSoftSSI, &ulDataRx[0]);
	SoftSSIDataGetNonBlocking(&g_sSoftSSI, &ulDataRx[3]);

	SysTickDisable();
#else

	// use hardware SPI
	while (SSIDataGetNonBlocking(SSI0_BASE, &ulDataRx[0]));		// read Rx FIFO until it's empty (ie dump junk)
	SSIDataPut(SSI0_BASE, (addr<<9)|0x0100);					// SPI: LSB of addr must be 1 for a read
	while(SSIBusy(SSI0_BASE));									// Wait until SSI0 is done transferring data
	SSIDataGetNonBlocking(SSI0_BASE, ulDataRx);					// read received data

#endif

	// UARTprintf("SPI_Read addr 0x%x: 0x%x  0x%x\n", addr, (int)ulDataRx[0], (int)ulDataRx[1]);

	return (((unsigned short)ulDataRx[0])&0xff);
}




//*****************************************************************************
//
// Counts the number of periods (decimal points) in the string. Is given a
// pointer to the string and the number of characters in the string that need to
// be checked.
//
// Technically speaking, the string might not actually be a "string" because it
// does not necessarily need to be null-terminated. We simply check the specified
// number of characters.
//
//*****************************************************************************

int numperiods (char * string, int len)
{
	int i, cnt = 0;

	if (len == 0)
		return 0;

	for (i=0; i<len; i++)
		if (string[i] == '.')
			cnt++;

	return cnt;
}


//*****************************************************************************
//
// Remove any trailing zeros from the end of the specified string. 'len' is the
// number of characters in the string NOT including any null at the end. Returns
// the new number of chars in the string. Does not remove the first character
// of the string (ie, a string containing only a single zero is allowed).
// And, it only does it if there's a decimal point in the string. For example,
// the string "400" needs to stay unchanged - "400" is not the same as "4"!
// However the string "400.000" should be changed to "400"
//
//*****************************************************************************
int RemoveTrailingZeros (char * string, int len)
{
	if (len < 2)
		return len;

	if (numperiods(string, len) == 0)
		return len;									// exit if no decimal point in string

	while ((string[len-1] == '0') && (len > 1))
		len--;										// remove trailing zeros from string

	if (string[len-1] == '.')
		len--;										// if end character is a decimal point, remove it

	string[len] = 0;								// place a null at the end of the string (to make it a real string again)

	return len;
}




//*****************************************************************************
//
// Sets output frequency of the LTC6946-3
//
// Calculates the various magic numbers required for its internal registers,
// then writes them to the part. Returns the calculated value of the
// output frequency (may not be the same as what was requested).
//
// If desired frequency is 0, mutes the output of the part.
//
// This function does NOT check that the requested frequency is valid. Providing
// an invalid frequency can produce unpredictable results.
//
// Remember that "freq" parameter is desired frequency divided by 4. Same for
// return value.
//
//*****************************************************************************
int SetOutputFreq (int freq)
{
	int R, N, O, BD, vco;

	if (freq == 0) {
		// frequency is zero, mute the LTC6946 as much as possible
		SPI_Write (2, 0xF0E);			// mute output
		return 0;
	}

	// Frequency not zero - calculate the magic numbers.

	// First we must determine O. This is selected from the table on the front page of the datasheet.
	// There is overlap in the ranges. In the case of overlap, we select the lowest possible value of O.
	if (freq < 192000000)
		O = 6;
	else if (freq < 240000000)
		O = 5;
	else if (freq < 320000000)
		O = 4;
	else if (freq < 480000000)
		O = 3;
	else if (freq < 723750000)
		O = 2;
	else
		O = 1;

	// Now we have O, the output divider, we can calculate what the internal VCO must be running at to
	// give us our output frequency. Because output_frequency = VCO / O
	vco = freq * O;					// vco is /4 because freq is /4

	// In theory we would use the smallest possible input frequency to the VCO (largest possible value of R)
	// however LTC recommends using the smallest value of R possible that satisfies the system constraints
	// (whatever that really means). R can be in the range 1 - 1023, however we choose a fixed value of 112.
	// With a 10 MHz input clock, this gives us 89.286 kHz as the input to the VCO, which can be multiplied
	// by N, maximum 65535, to achieve 5.85 GHz (where max VCO freq is 5.79 GHz).
	R = 112;
	N = (vco / (2500000 / R));

	// Note that with a fixed reference frequency of 10 MHz and a fixed R of 112, resulting in a fixed
	// output of 89.286 kHz; this is the value of f(PFD). From table 7 of the datasheet, this means
	// a value of 0 for BD.
	BD = 0;

	UARTprintf("O=%d R=%d N=%d\n", O, R, N);

	SPI_Write (2, 0x0E);						// mute output, chip running, no REF output

	SPI_Write (1, 4);							// LOCK indication on STAT output pin
	SPI_Write (3, (BD<<4)|((R&0x300)>>8));		// high 4 bits are BD, low 2 bits are the top 2 bits of R
	SPI_Write (4, R&0xFF);						// bottom 8 bits of R
	SPI_Write (5, (N&0xFF00)>>8);				// top 8 bits of N
	SPI_Write (6, N&0xFF);						// bottom 8 bits of N
	SPI_Write (7, 0xb1);						// calibration (CAL) bit is here; we set it later
	SPI_Write (8, 0xF8 | O);					// output divider
	SPI_Write (9, 0x8B);
	SPI_Write (0xA, 0);

	SPI_Write (7, 0xb3);						// initiate calibration (completes automatically), don't wait for it to finish

	// while (SPI_Read(7) & 0x02);					// wait for CAL bit to clear

	SPI_Write (2, 0x0C);						// unmute output, chip running, no REF output

	UARTprintf("N: 0x%x  reg 5: 0x%x  reg 6: 0x%x\n", N, SPI_Read(5), SPI_Read(6));


	// return with the calculated output freq (divided by 4) based upon the above divider and multiplier values
	return ((2500000/R)*(N/O));
}



//*****************************************************************************
//
// Converts the (potentially) floating-point number string entered by the user
// to an integer frequency, divided by 4. (Divide by 4 because otherwise 5.79 GHz
// will overflow a 32-bit integer.) Then check the number. If it's within the
// acceptable range, update the "current frequency" string with it, give it
// to the LTC6946, and return 0. If the frequency is not within the acceptable
// range, return non-zero.
//
// As an aside, I tried to use atof() to read the frequency string and then convert
// it. But it was inaccurate. For example 0.08 became 0.079. Simple code, but poor result.
// That's the nature of using floats.
//
// Format of the input string is always: "0 MHz" or "1.3 GHz" etc. There is always
// a space followed by a multiplier (MHz or GHz) even if the actual number is zero.
//
// Returns 0 if frequency update went OK, non-zero if there was a problem.
//
//*****************************************************************************
int UpdateFrequency (char * string, int len)
{
	int i_freq = 0, i, decpoint = 0, scaler;
	uint64 temp64;


	// Check we have at least the string "0 MHz", ie 5 characters. If not, something's
	// wrong / unexpected with this string.
	if (len < 5) {
		UARTprintf ("UpdateFrequency error: string length %d\n", len);
		return 1;
	}

	// Check to see if frequency is zero
	if ( (string[0] == '0') && (string[1] == 0))
		i_freq = 0;

	// frequency is not zero - let's process it
	else {
		// First we need to convert the potentially floating-point string number into an int. We do this
		// by working through the number backwards, taking note of the location of the decimal point (if present)
		// so that by looking at the decimal point location, and if it's GHz or MHz, we know what to multiply the
		// basic number by, to create the final int. With a string of format "xxx.xx MHz" the last digit will be
		// at location [len-5]
		for (i = len-4; i;) {
			i--;
			if (string[i] == '.')
				decpoint = len - 5 - i;				// number digits to right of decimal point
			else
				if (decpoint)
					i_freq += (string[i] - '0') * pow10[len-5-i-1];		// convert ascii character to an int
				else
					i_freq += (string[i] - '0') * pow10[len-5-i];		// convert ascii character to an int
		}

	}

	// UARTprintf("Freq = %d  DecPt = %d\n", i_freq, decpoint);

	// Now calculate what the frequency integer really is. Divided by 4 (because 5.79 GHz is too big a number to
	// fit into a 32-bit int). Basically we take the i_freq value, which is the integer number, and multiply by a
	// scaling factor that's a function of (a) the decimal point location, and (b) MHz or GHz.
	// Because the user could potentially enter a VERY large frequency (eg 100 GHz) this little bit of math is done
	// using 64-bit integers.
	scaler = (string[len-3] == 'G') ? 250000000 : 250000;			// this incorporates the divide-by-4
	temp64 = (uint64)i_freq * (uint64)scaler;
	// divide by 10 decimal-point times
	while (decpoint--)
		temp64 /= (uint64)10;

	// Good - we have a 64-bit integer frequency which is the user entry divided by 4
	// Check to see if it's within the allowable frequency range
	// There a min freq, a max freq, and also an un-generatable gap in the middle of the range
	if (temp64 > (uint64)MAX_FREQ)
		return 2;
	if ((temp64 < (uint64)MIN_FREQ) && (temp64 > (uint64)0))
		return 3;
	if ( (temp64 > (uint64)MIN_GAP_FREQ) && (temp64 < (uint64)MAX_GAP_FREQ) )
		return 4;

	// Very good - user entered frequency is within allowable range.
	i_freq = (int)temp64;						// user freq back in a more manageable 32-bit int

	// UARTprintf("Freq / 4 = %d\n", i_freq);

	// copy keypad frequency string into current frequency string, including null at end
	strcpy (s_frequency, string);

	// Now it's time to calculate some magic numbers for the LTC6946 internal registers,
	// then write them to the LTC6946.
	//
	// What would also be interesting is to use those magic numbers to calculate the actual
	// frequency being generated by the LTC6946 and display that. Because it may not be exactly
	// the frequency you asked for. We display these numbers in the printf to the serial port, but
	// not on the screen, because the actual freq is very close to the requested freq.
	// Eg: 2 GHz becomes 1.99996 GHz, an error of only 19 PPM.
	i = SetOutputFreq (i_freq);
	UARTprintf("freq/4: requested=%d actual=%d\n", i_freq, i);

	return 0;
}


//*****************************************************************************
//
// This function is called by the graphics library widget manager in the
// context of WidgetMessageQueueProcess whenever the user releases the
// "Set Frequency" button.  We use this notification to change to the next
// screen, screen 2, where the user can enter a new frequency with a keypad.
//
//*****************************************************************************
void OnSetFreqPress (tWidget *pWidget)
{
    // Play the sound effect associated with a key press.
    SoundPlay(g_pusKeyClick, sizeof(g_pusKeyClick) / 2);

    // Move on to screen 2, where the user can enter a new frequency with a keypad
   	WidgetRemove ((tWidget *)&g_s1Background);				// remove screen 1 (widget tree now empty)
   	WidgetAdd(WIDGET_ROOT, (tWidget *)&g_s2Background);		// add screen 2 widgets to widget tree
   	WidgetPaint((tWidget *)&g_s2Background);				// draw screen 2

   	strcpy (keypadbuff, "-");
   	charcount = 0;											// initialize the keypad "frequency" string
}



//*****************************************************************************
//
// Called when the user presses the "OK" button on screen 3 (the error screen).
// Simply change to screen 1 (the "current frequency" display screen).
//
//*****************************************************************************
void OnOKPress (tWidget *pWidget)
{
    // Play the sound effect associated with a key press.
    SoundPlay(g_pusKeyClick, sizeof(g_pusKeyClick) / 2);

    // Change to screen 1, the "current frequency" screen
   	WidgetRemove ((tWidget *)&g_s3Background);				// remove screen 3 (widget tree now empty)
   	WidgetAdd(WIDGET_ROOT, (tWidget *)&g_s1Background);		// add screen 1 widgets to widget tree
   	WidgetPaint((tWidget *)&g_s1Background);				// draw screen 1
}



//*****************************************************************************
//
// Called when the user presses a keypad button on screen 2 (the frequency
// entering screen).
//
//*****************************************************************************

void KeyPadPress (tWidget *pWidget)
{
    // Play the sound effect associated with a key press.
    SoundPlay(g_pusKeyClick, sizeof(g_pusKeyClick) / 2);

    // Check which keypad key was pressed, and handle it accordingly
    // I tried to use a switch statement, but the compiler needs constant arguments.
    // So we have a bunch of if - else if statements instead. Ah well.
    // (Yes, an array would be cleaner, but harder to read? I'm a simple but slow kinda guy.)

    // Start with the numbers 1 through 9.
    if (pWidget == (tWidget *)&g_s2Key1) {
    	if ((charcount==1) && (keypadbuff[0]=='0'))
    		charcount = 0;								// remove leading zero, if it exists
    	if (charcount < MAXCHARS)
    		keypadbuff[charcount++] = '1';
    }

    else if (pWidget == (tWidget *)&g_s2Key2) {
    	if ((charcount==1) && (keypadbuff[0]=='0'))
    		charcount = 0;								// remove leading zero, if it exists
    	if (charcount < MAXCHARS)
    		keypadbuff[charcount++] = '2';
    }

    else if (pWidget == (tWidget *)&g_s2Key3) {
    	if ((charcount==1) && (keypadbuff[0]=='0'))
    		charcount = 0;								// remove leading zero, if it exists
    	if (charcount < MAXCHARS)
    		keypadbuff[charcount++] = '3';
    }

    else if (pWidget == (tWidget *)&g_s2Key4) {
    	if ((charcount==1) && (keypadbuff[0]=='0'))
    		charcount = 0;								// remove leading zero, if it exists
    	if (charcount < MAXCHARS)
    		keypadbuff[charcount++] = '4';
    }

    else if (pWidget == (tWidget *)&g_s2Key5) {
    	if ((charcount==1) && (keypadbuff[0]=='0'))
    		charcount = 0;								// remove leading zero, if it exists
    	if (charcount < MAXCHARS)
    		keypadbuff[charcount++] = '5';
    }

    else if (pWidget == (tWidget *)&g_s2Key6) {
    	if ((charcount==1) && (keypadbuff[0]=='0'))
    		charcount = 0;								// remove leading zero, if it exists
    	if (charcount < MAXCHARS)
    		keypadbuff[charcount++] = '6';
    }

    else if (pWidget == (tWidget *)&g_s2Key7) {
    	if ((charcount==1) && (keypadbuff[0]=='0'))
    		charcount = 0;								// remove leading zero, if it exists
    	if (charcount < MAXCHARS)
    		keypadbuff[charcount++] = '7';
    }

    else if (pWidget == (tWidget *)&g_s2Key8) {
    	if ((charcount==1) && (keypadbuff[0]=='0'))
    		charcount = 0;								// remove leading zero, if it exists
    	if (charcount < MAXCHARS)
    		keypadbuff[charcount++] = '8';
    }

    else if (pWidget == (tWidget *)&g_s2Key9) {
    	if ((charcount==1) && (keypadbuff[0]=='0'))
    		charcount = 0;								// remove leading zero, if it exists
    	if (charcount < MAXCHARS)
    		keypadbuff[charcount++] = '9';
    }

    // Check the number zero. We don't allow more than 1 leading zero, and even then only if it's
    // followed by the decimal point. So if we're in the second location in the buffer AND the
    // first location contains a zero, we don't accept this zero.
    else if (pWidget == (tWidget *)&g_s2Key0) {
    	if ((charcount < MAXCHARS) && !((charcount==1) && (keypadbuff[0] == '0')) )
    		keypadbuff[charcount++] = '0';
    }

    // Check for the decimal point ("period") character. We accept it anywhere, including being the
    // first character. But we only accept one, so we don't accept it if there's already one in the string.
    // And, if it's the first character, we'll automatically prefix it with a zero, so it reads better.
    else if (pWidget == (tWidget *)&g_s2KeyPeriod) {
    	if ( (charcount < MAXCHARS) && (numperiods(keypadbuff, charcount) == 0) ) {
    		if (charcount == 0) {
    			keypadbuff[0] = '0';					// this period is first character in string, so preface
    			keypadbuff[1] = '.';					// it with a zero
    			charcount = 2;
    		}
    		else
    			keypadbuff[charcount++] = '.';
    	}
    }

    // Handle the backspace character. Simply moves one character backwards in the string, effectively deleting
    // whatever character was at the end of the string.
    else if (pWidget == (tWidget *)&g_s2KeyBackspace) {
    	if (charcount > 0)
    		charcount--;
    }

    // Handle the cancel key. Go back to screen 1 (the "current frequency" display screen) without
    // altering the current frequency.
    else if (pWidget == (tWidget *)&g_s2KeyCancel) {
    	WidgetRemove ((tWidget *)&g_s2Background);				// remove screen 2 (widget tree now empty)
    	WidgetAdd(WIDGET_ROOT, (tWidget *)&g_s1Background);		// add screen 1 widgets to widget tree
    	WidgetPaint((tWidget *)&g_s1Background);				// draw screen 1
    	return;
    }


    // Handle the MHz and GHz keys. These are very similar. They just add a "MHz" or "GHz" to the end
    // of the keypad frequency string, then call a function which checks the frequency. If
    // there's a problem with the entered frequency (ie it's outside the allowable range) we display the
    // error message screen (screen 3). If the entered frequency is OK we change to the new frequency and
    // display the "current frequency" screen, screen 1. Either way, we're changing screens.
    else if ((pWidget == (tWidget *)&g_s2KeyMHz) || (pWidget == (tWidget *)&g_s2KeyGHz)) {
    	if (charcount == 0) {
    		keypadbuff[0] = '0';
    		keypadbuff[1] = 0;										// Handle case where user presses "MHz" or "GHz"
    		charcount = 1;											//  without entering a number first. Make it zero.
    	}
    	charcount = RemoveTrailingZeros (keypadbuff, charcount);	// remove any excess zeros from keypad freq string
    	if (pWidget == (tWidget *)&g_s2KeyMHz)
    		strcpy (&keypadbuff[charcount], " MHz");
    	else
    		strcpy (&keypadbuff[charcount], " GHz");
    	charcount += 4;												// append " MHz" or " GHz" to end of keypad freq string

    	WidgetRemove ((tWidget *)&g_s2Background);					// remove screen 2 (widget tree now empty)
    	if (UpdateFrequency (keypadbuff, charcount)) {
    		// problem updating frequency, display error screen, screen 3
        	WidgetAdd(WIDGET_ROOT, (tWidget *)&g_s3Background);		// add screen 3 widgets to widget tree
        	WidgetPaint((tWidget *)&g_s3Background);				// draw screen 3
    	}
    	else {
    		// frequency updated ok, display "current frequency" screen, screen 1
        	WidgetAdd(WIDGET_ROOT, (tWidget *)&g_s1Background);		// add screen 1 widgets to widget tree
        	WidgetPaint((tWidget *)&g_s1Background);				// draw screen 1
    	}
    	return;
    }


    // should never execute here - means a non-existent key was pressed. Just ignore it - do nothing
    else {
    }


    // update display of the string of user-entered characters
    keypadbuff[charcount] = 0;						// null-terminate current string
    WidgetPaint((tWidget *)&g_s2UserEntry);			// redraw display
}


//*****************************************************************************
//
// Initialise the UART at 115kb/s
//
//*****************************************************************************
void UartInit (void)
{
    //
    // Enable the peripherals
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);

    //
    // Configure the pin muxing for UART0 functions on port A0 and A1.
    //
    // GPIOPinConfigure(GPIO_PA0_U0RX);
    // GPIOPinConfigure(GPIO_PA1_U0TX);

    //
    // Configure the UART pins appropriately.
    //
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_1 | GPIO_PIN_0);

    //
    // Initialize the UART as a console for text I/O.
    //
    UARTStdioInit(0);

    //
    // Print hello message out the uart
    //
    UARTprintf("\nLTC6946 Signal Generator\n");
}



//*****************************************************************************
//
// Main Loop
//
//*****************************************************************************
int main(void)
{
    //
    // Set the system clock to run at 25MHz from the PLL
    //
    SysCtlClockSet(SYSCTL_SYSDIV_8 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                       SYSCTL_XTAL_8MHZ);

    //
    // Enable Interrupts
    //
    IntMasterEnable();

    //
    // Initialize the display driver.
    //
    Kitronix320x240x16_SSD2119Init();

    //
    // Turn on the display backlight at full brightness.
    //
    Kitronix320x240x16_SSD2119BacklightOn(255);

    //
    // Initialize the UART (so we can print stuff out on it)
    //
    UartInit();

    //
    // Init the SPI, used to talk to the LTC6946
    //
    Init_SPI();

    // With the SPI initialized, we can set the LTC6946 output frequency
    SetOutputFreq (0);

    // Read register 0x0b from the LTC6946. It should return 0x43
    if (SPI_Read(0x0b) != 0x43) {
    	UARTprintf("\n\n*** ERROR reading register 0x0b from LTC6946 ***\n\n");
    	SetOutputFreq (0);
    }

    //
    // Initialize the sound driver. Makes a beep when a keypad button is pressed.
    //
    SoundInit();

    //
    // Initialize the touch screen driver.
    //
    TouchScreenInit();

    //
    // Set the touch screen event handler.
    //
    TouchScreenCallbackSet(WidgetPointerMessage);

    // Initialize the "current frequency" string
    strcpy (s_frequency, "0 Hz");

    //
    // Add the compile-time defined widgets for screen 1 to the widget tree.
    //
    WidgetAdd(WIDGET_ROOT, (tWidget *)&g_s1Background);

    //
    // Paint the widget tree to display the first (default) screen on the LCD
    //
    WidgetPaint ((tWidget *)&g_s1Background);

    //
    // Loop forever, processing widget messages.
    //
    while(1)
    {
        //
        // Process any messages from or for the widgets.
        //
        WidgetMessageQueueProcess();
    }

}
